
// Code from http://www.generative-gestaltung.de

/**
 * more nodes and more springs
 *
 * KEYS
 * r             : reset positions
 * s             : save png
 * p             : save pdf
 */

import processing.pdf.*;
import java.util.Calendar;

boolean savePDF = false;
float nodeDiameter = 16;


// an array for the nodes
Node[] nodes = new Node[100];
// an array for the springs
Spring[] springs = new Spring[0];

// dragged node
Node selectedNode = null;



void setup() {
  size(700, 700);
  smooth();
  noStroke();
  initNodesAndSprings();
}


void draw() {
  if (savePDF) beginRecord(PDF, timestamp()+".pdf");

  background(255);
  // with reverberation
  //fill(0, 0, 0, 10);
  //rect(0, 0, width, height);

  // apply repell forces among nodes
  for (int i = 0 ; i < nodes.length; i++) {
    nodes[i].attract(nodes);
  } 
  
  // apply spring forces among nodes
  for (int i = 0 ; i < springs.length; i++) {
    springs[i].update();
  } 

  // update positions of nodes
  for (int i = 0 ; i < nodes.length; i++) {
    nodes[i].update();
  } 

  // draw springs
  for (int i = 0 ; i < springs.length; i++) {
    springs[i].show();
  }
  
  // draw nodes
  for (int i = 0 ; i < nodes.length; i++) {
    nodes[i].show(nodeDiameter);  
  }

  if (selectedNode != null) {
    selectedNode.x = mouseX;
    selectedNode.y = mouseY;
  }

  if (savePDF) {
    savePDF = false;
    println("saving to pdf – finishing");
    endRecord();
  }
}


void initNodesAndSprings() {
  // init nodes
  float rad = nodeDiameter/2;
  for (int i = 0; i < nodes.length; i++) {
    nodes[i] = new Node(width/2+random(-200, 200), height/2+random(-200, 200));
    nodes[i].setBoundary(rad, rad, width-rad, height-rad);
    nodes[i].setRadius(100);
    nodes[i].setStrength(-5);
  } 

  // set springs randomly
  springs = new Spring[0];

  // This parameter influences the density of the graph
  int density = 1;
  for (int j = 0 ; j < nodes.length-1; j++) {
    int rCount = floor(random(1, density+1));
    for (int i = 0; i < rCount; i++) {
      // this rule generates trees (connected acyclic graphs)
      int r = floor(random(j+1, nodes.length));
      // this rule generates graphs that can be cyclic (including self-loops) and not connected 
      //int r = floor(random(0, nodes.length));
      Spring newSpring = new Spring(nodes[j], nodes[r]);
      // force edges with the same length
      newSpring.setLength(20);
      // speed of convergenge to the final layout
      newSpring.setStiffness(1);
      springs = (Spring[]) append(springs, newSpring);
    }
  }
}


void mousePressed() {
  // Ignore anything greater than this distance
  float maxDist = 20;
  for (int i = 0; i < nodes.length; i++) {
    Node checkNode = nodes[i];
    float d = dist(mouseX, mouseY, checkNode.x, checkNode.y);
    if (d < maxDist) {
      selectedNode = checkNode;
      maxDist = d;
    }
  }
}

void mouseReleased() {
  if (selectedNode != null) {
    selectedNode = null;
  }
}


void keyPressed() {
  if(key=='s' || key=='S') saveFrame(timestamp()+"_##.png"); 

  if(key=='p' || key=='P') {
    savePDF = true; 
    println("saving to pdf - starting (this may take some time)");
  }

  if(key=='r' || key=='R') {
    background(255);
    initNodesAndSprings();
  }
}


String timestamp() {
  return String.format("%1$ty%1$tm%1$td_%1$tH%1$tM%1$tS", Calendar.getInstance());
}